@@ -401,7 +401,7 @@ gdm_slave_connect_to_x11_display (GdmSlave *slave)
         if (slave->priv->server_display == NULL) {
                 g_warning ("Unable to connect to display %s", slave->priv->display_name);
                 ret = FALSE;
-        } else {
+        } else if (slave->priv->display_is_local) {
                 XHostAddress host_entries[2] = {
                         { FamilyServerInterpreted },
                         { FamilyServerInterpreted }
@@ -431,6 +431,9 @@ gdm_slave_connect_to_x11_display (GdmSlave *slave)
 
                 XAddHosts (slave->priv->server_display, host_entries,
                            G_N_ELEMENTS (host_entries));
+        } else {
+                g_debug ("GdmSlave: Connected to display %s", slave->priv->display_name);
+                ret = TRUE;
         }
 
         return ret;
@@ -789,7 +792,6 @@ gdm_slave_parse_enriched_login (GdmSlave *slave,
                 char     **argv;
                 char      *std_output;
                 char      *std_error;
-                int       *exit_status;
 
                 /* Remove the pipe symbol */
                 username[username_length-1] = '\0';
@@ -814,7 +816,7 @@ gdm_slave_parse_enriched_login (GdmSlave *slave,
                                     NULL,
                                     &std_output,
                                     &std_error,
-                                    exit_status,
+                                    NULL,
                                     &error);
 
                 g_ptr_array_foreach (env, (GFunc)g_free, NULL);
@@ -836,6 +838,8 @@ gdm_slave_parse_enriched_login (GdmSlave *slave,
         } else {
                 return (g_strdup (username));
         }
+
+	return NULL;
 }
 
 gboolean
@@ -881,6 +885,16 @@ gdm_slave_get_timed_login_details (GdmSlave   *slave,
         if (usernamep != NULL) {
                 *usernamep = gdm_slave_parse_enriched_login (slave,
                         username, slave->priv->display_name);
+        } else {
+                g_free (username);
+
+                if (enabledp != NULL) {
+                        *enabledp = enabled;
+                }
+                if (delayp != NULL) {
+                        *delayp = delay;
+                }
+                return TRUE;
         }
         g_free (username);
 
