@@ -104,7 +104,8 @@ struct GdmChooserWidgetPrivate
 enum {
         PROP_0,
         PROP_INACTIVE_TEXT,
-        PROP_ACTIVE_TEXT
+        PROP_ACTIVE_TEXT,
+        PROP_LIST_VISIBLE
 };
 
 enum {
@@ -275,6 +276,26 @@ translate_list_path_to_view_path (GdmChooserWidget  *widget,
         *path = sorted_path;
 }
 
+
+static void
+translate_view_path_to_list_path (GdmChooserWidget  *widget,
+                                  GtkTreePath      **path)
+{
+        GtkTreePath *filtered_path;
+        GtkTreePath *list_path;
+
+        /* the child model is the source for the filter */
+        filtered_path = gtk_tree_model_sort_convert_path_to_child_path (widget->priv->model_sorter,
+                                                                        *path);
+
+        list_path = gtk_tree_model_filter_convert_path_to_child_path (widget->priv->model_filter,
+                                                                      filtered_path);
+        gtk_tree_path_free (filtered_path);
+
+        gtk_tree_path_free (*path);
+        *path = list_path;
+}
+
 static GtkTreePath *
 get_list_path_to_active_row (GdmChooserWidget *widget)
 {
@@ -335,7 +356,7 @@ get_active_item_id (GdmChooserWidget *widget,
                                     CHOOSER_ID_COLUMN,
                                     &item_id,
                                     -1);
-        };
+        }
         gtk_tree_path_free (path);
 
         return item_id;
@@ -349,6 +370,105 @@ gdm_chooser_widget_get_active_item (GdmChooserWidget *widget)
         return get_active_item_id (widget, &iter);
 }
 
+static void
+get_selected_list_path (GdmChooserWidget *widget,
+                        GtkTreePath     **pathp)
+{
+        GtkTreeSelection    *selection;
+        GtkTreeModel        *sort_model;
+        GtkTreeIter          sorted_iter;
+        GtkTreePath         *path;
+
+        path = NULL;
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget->priv->items_view));
+        if (gtk_tree_selection_get_selected (selection, &sort_model, &sorted_iter)) {
+
+                g_assert (sort_model == GTK_TREE_MODEL (widget->priv->model_sorter));
+
+                path = gtk_tree_model_get_path (sort_model, &sorted_iter);
+
+                translate_view_path_to_list_path (widget, &path);
+        } else {
+                g_debug ("GdmChooserWidget: no rows selected");
+        }
+
+        *pathp = path;
+}
+
+char *
+gdm_chooser_widget_get_selected_item (GdmChooserWidget *widget)
+{
+        GtkTreeIter   iter;
+        GtkTreeModel *model;
+        GtkTreePath  *path;
+        char         *id;
+
+        id = NULL;
+
+        get_selected_list_path (widget, &path);
+
+        if (path == NULL) {
+                return NULL;
+        }
+
+        model = GTK_TREE_MODEL (widget->priv->list_store);
+
+        if (gtk_tree_model_get_iter (model, &iter, path)) {
+                gtk_tree_model_get (model,
+                                    &iter,
+                                    CHOOSER_ID_COLUMN,
+                                    &id,
+                                    -1);
+        }
+
+        gtk_tree_path_free (path);
+
+        return id;
+}
+
+void
+gdm_chooser_widget_set_selected_item (GdmChooserWidget *widget,
+                                      const char       *id)
+{
+        GtkTreeIter       iter;
+        GtkTreeSelection *selection;
+        GtkTreeModel     *model;
+
+        g_return_if_fail (GDM_IS_CHOOSER_WIDGET (widget));
+
+        g_debug ("GdmChooserWidget: setting selected item '%s'",
+                 id ? id : "(null)");
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget->priv->items_view));
+
+        model = GTK_TREE_MODEL (widget->priv->list_store);
+
+        if (find_item (widget, id, &iter)) {
+                GtkTreePath  *path;
+
+                path = gtk_tree_model_get_path (model, &iter);
+                translate_list_path_to_view_path (widget, &path);
+
+                gtk_tree_selection_select_path (selection, path);
+
+                gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (widget->priv->items_view),
+                                              path,
+                                              NULL,
+                                              TRUE,
+                                              0.5,
+                                              0.0);
+
+                gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget->priv->items_view),
+                                          path,
+                                          NULL,
+                                          FALSE);
+                gtk_tree_path_free (path);
+        } else {
+                gtk_tree_selection_unselect_all (selection);
+        }
+}
+
 static void
 activate_from_item_id (GdmChooserWidget *widget,
                        const char       *item_id)
@@ -477,6 +597,17 @@ update_separator_visibility (GdmChooserWidget *widget)
                             -1);
 }
 
+static void
+update_chooser_visibility (GdmChooserWidget *widget)
+{
+        if (gdm_chooser_widget_get_number_of_items (widget) > 0) {
+                gtk_widget_show (widget->priv->scrollable_widget);
+        } else {
+                gtk_widget_hide (widget->priv->scrollable_widget);
+        }
+        g_object_notify (G_OBJECT (widget), "list-visible");
+}
+
 static void
 set_inactive_items_visible (GdmChooserWidget *widget,
                             gboolean          should_show)
@@ -920,51 +1051,6 @@ deactivate (GdmChooserWidget *widget)
         g_signal_emit (widget, signals[DEACTIVATED], 0);
 }
 
-static void
-translate_view_path_to_list_path (GdmChooserWidget  *widget,
-                                  GtkTreePath      **path)
-{
-        GtkTreePath *filtered_path;
-        GtkTreePath *list_path;
-
-        /* the child model is the source for the filter */
-        filtered_path = gtk_tree_model_sort_convert_path_to_child_path (widget->priv->model_sorter,
-                                                                        *path);
-
-        list_path = gtk_tree_model_filter_convert_path_to_child_path (widget->priv->model_filter,
-                                                                      filtered_path);
-        gtk_tree_path_free (filtered_path);
-
-        gtk_tree_path_free (*path);
-        *path = list_path;
-}
-
-static void
-get_selected_list_path (GdmChooserWidget *widget,
-                        GtkTreePath     **pathp)
-{
-        GtkTreeSelection    *selection;
-        GtkTreeModel        *sort_model;
-        GtkTreeIter          sorted_iter;
-        GtkTreePath         *path;
-
-        path = NULL;
-
-        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget->priv->items_view));
-        if (gtk_tree_selection_get_selected (selection, &sort_model, &sorted_iter)) {
-
-                g_assert (sort_model == GTK_TREE_MODEL (widget->priv->model_sorter));
-
-                path = gtk_tree_model_get_path (sort_model, &sorted_iter);
-
-                translate_view_path_to_list_path (widget, &path);
-        } else {
-                g_debug ("GdmChooserWidget: no rows selected");
-        }
-
-        *pathp = path;
-}
-
 void
 gdm_chooser_widget_activate_selected_item (GdmChooserWidget *widget)
 {
@@ -1013,7 +1099,8 @@ gdm_chooser_widget_set_active_item (GdmChooserWidget *widget,
 {
         g_return_if_fail (GDM_IS_CHOOSER_WIDGET (widget));
 
-        g_debug ("GdmChooserWidget: setting active item '%s'", id);
+        g_debug ("GdmChooserWidget: setting active item '%s'",
+                 id ? id : "(null)");
 
         if (id != NULL) {
                 activate_from_item_id (widget, id);
@@ -1076,6 +1163,9 @@ gdm_chooser_widget_get_property (GObject        *object,
         case PROP_ACTIVE_TEXT:
                 g_value_set_string (value, self->priv->active_text);
                 break;
+        case PROP_LIST_VISIBLE:
+                g_value_set_boolean (value, GTK_WIDGET_VISIBLE (self->priv->scrollable_widget));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -1248,6 +1338,14 @@ gdm_chooser_widget_class_init (GdmChooserWidgetClass *klass)
                                                               (G_PARAM_READWRITE |
                                                                G_PARAM_CONSTRUCT)));
 
+        g_object_class_install_property (object_class,
+                                         PROP_LIST_VISIBLE,
+                                         g_param_spec_boolean ("list-visible",
+                                                              _("List Visible"),
+                                                              _("Whether or not the chooser list is visible"),
+                                                              TRUE,
+                                                              G_PARAM_READABLE));
+
         g_type_class_add_private (klass, sizeof (GdmChooserWidgetPrivate));
 }
 
@@ -1313,6 +1411,8 @@ compare_item  (GtkTreeModel *model,
         GdmChooserWidget *widget;
         char             *name_a;
         char             *name_b;
+        char             *text_a;
+        char             *text_b;
         gulong            prio_a;
         gulong            prio_b;
         gboolean          is_separate_a;
@@ -1321,6 +1421,7 @@ compare_item  (GtkTreeModel *model,
         int               direction;
         GtkTreeIter      *separator_iter;
         char             *id;
+        PangoAttrList    *attrs;
 
         g_assert (GDM_IS_CHOOSER_WIDGET (data));
 
@@ -1380,7 +1481,14 @@ compare_item  (GtkTreeModel *model,
                 result *= direction;
         } else if (is_separate_b == is_separate_a) {
                 if (prio_a == prio_b) {
-                        result = g_utf8_collate (name_a, name_b);
+                        pango_parse_markup (name_a, -1, 0, &attrs, &text_a, NULL, NULL);
+                        pango_parse_markup (name_b, -1, 0, &attrs, &text_b, NULL, NULL);
+                        if (text_a && text_b)
+                            result = g_utf8_collate (text_a, text_b);
+                        else
+                            result = g_utf8_collate (name_a, name_b);
+                        g_free (text_a);
+                        g_free (text_b);
                 } else if (prio_a > prio_b) {
                         result = -1;
                 } else {
@@ -1984,6 +2092,7 @@ gdm_chooser_widget_add_item (GdmChooserWidget *widget,
                                            -1);
 
         move_cursor_to_top (widget);
+        update_chooser_visibility (widget);
 }
 
 void
@@ -2032,6 +2141,7 @@ gdm_chooser_widget_remove_item (GdmChooserWidget *widget,
         gtk_list_store_remove (widget->priv->list_store, &iter);
 
         move_cursor_to_top (widget);
+        update_chooser_visibility (widget);
 }
 
 gboolean
