@@ -77,6 +77,7 @@ enum {
         LANGUAGE_SELECTED,
         LAYOUT_SELECTED,
         SESSION_SELECTED,
+        DIALOG_HIDDEN,
         NUMBER_OF_SIGNALS
 };
 
@@ -456,6 +457,17 @@ gdm_greeter_panel_class_init (GdmGreeterPanelClass *klass)
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
 
+        signals[DIALOG_HIDDEN] =
+                g_signal_new ("dialog-hidden",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdmGreeterPanelClass, dialog_hidden),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+
         g_object_class_install_property (object_class,
                                          PROP_MONITOR,
                                          g_param_spec_int ("monitor",
@@ -487,6 +499,14 @@ on_language_activated (GdmLanguageOptionWidget *widget,
         g_free (language);
 }
 
+static void
+on_language_dialog_hidden(GdmLanguageOptionWidget *widget,
+                          GdmGreeterPanel         *panel)
+{
+
+        g_signal_emit (panel, signals[DIALOG_HIDDEN], 0);
+}
+
 static void
 on_layout_activated (GdmLayoutOptionWidget *widget,
                      GdmGreeterPanel       *panel)
@@ -507,6 +527,15 @@ on_layout_activated (GdmLayoutOptionWidget *widget,
 
         g_free (layout);
 }
+
+static void
+on_layout_dialog_hidden (GdmLayoutOptionWidget *widget,
+                         GdmGreeterPanel         *panel)
+{
+
+        g_signal_emit (panel, signals[DIALOG_HIDDEN], 0);
+}
+
 static void
 on_session_activated (GdmSessionOptionWidget *widget,
                       GdmGreeterPanel        *panel)
@@ -580,6 +609,9 @@ gdm_greeter_panel_init (GdmGreeterPanel *panel)
         g_signal_connect (G_OBJECT (panel->priv->language_option_widget),
                           "language-activated",
                           G_CALLBACK (on_language_activated), panel);
+        g_signal_connect (G_OBJECT (panel->priv->language_option_widget),
+                          "dialog-hidden",
+                          G_CALLBACK (on_language_dialog_hidden), panel);
         gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), panel->priv->language_option_widget, FALSE, FALSE, 6);
         gdm_profile_end ("creating option widget");
 
@@ -587,6 +619,9 @@ gdm_greeter_panel_init (GdmGreeterPanel *panel)
         g_signal_connect (G_OBJECT (panel->priv->layout_option_widget),
                           "layout-activated",
                           G_CALLBACK (on_layout_activated), panel);
+        g_signal_connect (G_OBJECT (panel->priv->layout_option_widget),
+                          "dialog-hidden",
+                          G_CALLBACK (on_layout_dialog_hidden), panel);
         gtk_box_pack_start (GTK_BOX (panel->priv->option_hbox), panel->priv->layout_option_widget, FALSE, FALSE, 6);
 
         panel->priv->session_option_widget = gdm_session_option_widget_new ();
@@ -723,6 +758,20 @@ gdm_greeter_panel_set_default_layout_name (GdmGreeterPanel *panel,
 #ifdef HAVE_LIBXKLAVIER
         g_return_if_fail (GDM_IS_GREETER_PANEL (panel));
 
+        if (layout_name != NULL &&
+            !gdm_layout_is_valid (layout_name)) {
+                const char *default_layout;
+
+                default_layout = gdm_layout_get_default_layout ();
+
+                g_debug ("GdmGreeterPanel: default layout %s is invalid, resetting to: %s",
+                         layout_name, default_layout ? default_layout : "null");
+
+                g_signal_emit (panel, signals[LAYOUT_SELECTED], 0, default_layout);
+
+                layout_name = default_layout;
+        }
+
         if (layout_name != NULL &&
             !gdm_option_widget_lookup_item (GDM_OPTION_WIDGET (panel->priv->layout_option_widget),
                                             layout_name, NULL, NULL, NULL)) {
