@@ -41,13 +41,12 @@
 #define MAX_ICON_SIZE     128
 #define MAX_FILE_SIZE     65536
 #define MINIMAL_UID       100
-#define RELAX_GROUP       TRUE
-#define RELAX_OTHER       TRUE
 
 enum {
         PROP_0,
         PROP_MANAGER,
         PROP_REAL_NAME,
+        PROP_DISPLAY_NAME,
         PROP_USER_NAME,
         PROP_UID,
         PROP_HOME_DIR,
@@ -69,6 +68,7 @@ struct _GdmUser {
         uid_t           uid;
         char           *user_name;
         char           *real_name;
+        char           *display_name;
         char           *home_dir;
         char           *shell;
         GList          *sessions;
@@ -207,6 +207,9 @@ gdm_user_get_property (GObject    *object,
         case PROP_REAL_NAME:
                 g_value_set_string (value, user->real_name);
                 break;
+        case PROP_DISPLAY_NAME:
+                g_value_set_string (value, user->display_name);
+                break;
         case PROP_HOME_DIR:
                 g_value_set_string (value, user->home_dir);
                 break;
@@ -253,6 +256,14 @@ gdm_user_class_init (GdmUserClass *class)
                                                               NULL,
                                                               G_PARAM_READABLE));
 
+        g_object_class_install_property (gobject_class,
+                                         PROP_DISPLAY_NAME,
+                                         g_param_spec_string ("display-name",
+                                                              "Display Name",
+                                                              "The unique name to display for this user.",
+                                                              NULL,
+                                                              G_PARAM_READABLE));
+
         g_object_class_install_property (gobject_class,
                                          PROP_UID,
                                          g_param_spec_ulong ("uid",
@@ -370,6 +381,7 @@ gdm_user_init (GdmUser *user)
         user->manager = NULL;
         user->user_name = NULL;
         user->real_name = NULL;
+        user->display_name = NULL;
         user->sessions = NULL;
 }
 
@@ -384,6 +396,7 @@ gdm_user_finalize (GObject *object)
 
         g_free (user->user_name);
         g_free (user->real_name);
+        g_free (user->display_name);
 
         if (G_OBJECT_CLASS (gdm_user_parent_class)->finalize)
                 (*G_OBJECT_CLASS (gdm_user_parent_class)->finalize) (object);
@@ -412,13 +425,16 @@ _gdm_user_update (GdmUser             *user,
         /* Display Name */
         if (pwent->pw_gecos && pwent->pw_gecos[0] != '\0') {
                 gchar *first_comma;
+                gchar *real_name_utf8;
 
-                first_comma = strchr (pwent->pw_gecos, ',');
+                real_name_utf8 = g_locale_to_utf8 (pwent->pw_gecos, -1, NULL, NULL, NULL);
+
+                first_comma = strchr (real_name_utf8, ',');
                 if (first_comma) {
-                        real_name = g_strndup (pwent->pw_gecos,
-                                                  (first_comma - pwent->pw_gecos));
+                        real_name = g_strndup (real_name_utf8, first_comma - real_name_utf8);
+                        g_free (real_name_utf8);
                 } else {
-                        real_name = g_strdup (pwent->pw_gecos);
+                        real_name = real_name_utf8;
                 }
 
                 if (real_name[0] == '\0') {
@@ -441,6 +457,16 @@ _gdm_user_update (GdmUser             *user,
                 g_free (real_name);
         }
 
+        /* Unique Display Name */
+        if ((!user->real_name && user->display_name) ||
+            (user->real_name &&
+             user->display_name &&
+             strncmp (user->real_name, user->display_name, strlen (user->real_name)) != 0)) {
+                g_free (user->display_name);
+                user->display_name = NULL;
+                g_object_notify (G_OBJECT (user), "display-name");
+        }
+
         /* UID */
         if (pwent->pw_uid != user->uid) {
                 user->uid = pwent->pw_uid;
@@ -539,6 +565,26 @@ gdm_user_get_real_name (GdmUser *user)
         return (user->real_name ? user->real_name : user->user_name);
 }
 
+/**
+ * gdm_user_get_display_name:
+ * @user: the user object to examine.
+ *
+ * Retrieves the unique display name of @user.
+ *
+ * Returns: a pointer to an array of characters which must not be modified or
+ *  freed, or %NULL.
+ *
+ * Since: 1.0
+ **/
+G_CONST_RETURN gchar *
+gdm_user_get_display_name (GdmUser *user)
+{
+        g_return_val_if_fail (GDM_IS_USER (user), NULL);
+
+        return (user->display_name ? user->display_name
+                : gdm_user_get_real_name (user));
+}
+
 /**
  * gdm_user_get_user_name:
  * @user: the user object to examine.
@@ -607,6 +653,62 @@ gdm_user_get_login_frequency (GdmUser *user)
         return user->login_frequency;
 }
 
+/**
+ * _gdm_user_show_full_display_name:
+ * @user: the user object to examine.
+ *
+ * Updates the unique display name of @user to "Real Name (username)".
+ *
+ * Since: 1.0
+ **/
+void
+_gdm_user_show_full_display_name (GdmUser *user)
+{
+        char *uniq_name;
+
+        g_return_if_fail (GDM_IS_USER (user));
+
+        if (user->real_name != NULL) {
+                uniq_name = g_strdup_printf ("%s (%s)",
+                                             user->real_name,
+                                             user->user_name);
+        } else {
+                uniq_name = NULL;
+        }
+
+        if ((user->real_name && !user->display_name) ||
+            (!user->real_name && user->display_name) ||
+            (user->real_name &&
+             user->display_name &&
+             strcmp (uniq_name, user->display_name) != 0)) {
+                g_free (user->display_name);
+                user->display_name = uniq_name;
+                g_object_notify (G_OBJECT (user), "display-name");
+        } else {
+                g_free (uniq_name);
+        }
+}
+
+/**
+ * _gdm_user_show_short_display_name:
+ * @user: the user object to examine.
+ *
+ * Resets the unique display name of @user.
+ *
+ * Since: 1.0
+ **/
+void
+_gdm_user_show_short_display_name (GdmUser *user)
+{
+        g_return_if_fail (GDM_IS_USER (user));
+
+        if (user->display_name) {
+                g_free (user->display_name);
+                user->display_name = NULL;
+                g_object_notify (G_OBJECT (user), "display-name");
+        }
+}
+
 int
 gdm_user_collate (GdmUser *user1,
                   GdmUser *user2)
@@ -616,8 +718,8 @@ gdm_user_collate (GdmUser *user1,
         gulong      num1;
         gulong      num2;
 
-        g_return_val_if_fail (user1 == NULL || GDM_IS_USER (user1), 0);
-        g_return_val_if_fail (user2 == NULL || GDM_IS_USER (user2), 0);
+        g_return_val_if_fail (GDM_IS_USER (user1), 0);
+        g_return_val_if_fail (GDM_IS_USER (user2), 0);
 
         if (user1->real_name != NULL) {
                 str1 = user1->real_name;
@@ -660,10 +762,7 @@ gdm_user_collate (GdmUser *user1,
 
 static gboolean
 check_user_file (const char *filename,
-                 uid_t       user,
-                 gssize      max_file_size,
-                 gboolean    relax_group,
-                 gboolean    relax_other)
+                 gssize      max_file_size)
 {
         struct stat fileinfo;
 
@@ -673,31 +772,19 @@ check_user_file (const char *filename,
 
         /* Exists/Readable? */
         if (stat (filename, &fileinfo) < 0) {
+                g_debug ("File does not exist");
                 return FALSE;
         }
 
         /* Is a regular file */
         if (G_UNLIKELY (!S_ISREG (fileinfo.st_mode))) {
-                return FALSE;
-        }
-
-        /* Owned by user? */
-        if (G_UNLIKELY (fileinfo.st_uid != user)) {
-                return FALSE;
-        }
-
-        /* Group not writable or relax_group? */
-        if (G_UNLIKELY ((fileinfo.st_mode & S_IWGRP) == S_IWGRP && !relax_group)) {
-                return FALSE;
-        }
-
-        /* Other not writable or relax_other? */
-        if (G_UNLIKELY ((fileinfo.st_mode & S_IWOTH) == S_IWOTH && !relax_other)) {
+                g_debug ("File is not a regular file");
                 return FALSE;
         }
 
         /* Size is kosher? */
         if (G_UNLIKELY (fileinfo.st_size > max_file_size)) {
+                g_debug ("File is too large");
                 return FALSE;
         }
 
@@ -738,133 +825,29 @@ get_filesystem_type (const char *path)
 }
 
 static GdkPixbuf *
-render_icon_from_home (GdmUser *user,
-                       int      icon_size)
+render_icon_from_cache (GdmUser *user,
+                        int      icon_size)
 {
         GdkPixbuf  *retval;
         char       *path;
-        gboolean    is_local;
         gboolean    is_autofs;
         gboolean    res;
         char       *filesystem_type;
 
-        is_local = FALSE;
-
-        /* special case: look at parent of home to detect autofs
-           this is so we don't try to trigger an automount */
-        path = g_path_get_dirname (user->home_dir);
-        filesystem_type = get_filesystem_type (path);
-        is_autofs = (filesystem_type != NULL && strcmp (filesystem_type, "autofs") == 0);
-        g_free (filesystem_type);
-        g_free (path);
-
-        if (is_autofs) {
-                return NULL;
-        }
-
-        /* now check that home dir itself is local */
-        filesystem_type = get_filesystem_type (user->home_dir);
-        is_local = ((filesystem_type != NULL) &&
-                    (strcmp (filesystem_type, "nfs") != 0) &&
-                    (strcmp (filesystem_type, "afs") != 0) &&
-                    (strcmp (filesystem_type, "autofs") != 0) &&
-                    (strcmp (filesystem_type, "unknown") != 0) &&
-                    (strcmp (filesystem_type, "ncpfs") != 0));
-        g_free (filesystem_type);
-
-        /* only look at local home directories so we don't try to
-           read from remote (e.g. NFS) volumes */
-        if (! is_local) {
-                return NULL;
-        }
-
-        /* First, try "~/.face" */
-        path = g_build_filename (user->home_dir, ".face", NULL);
+        path = g_build_filename (GDM_CACHE_DIR, user->user_name, "face", NULL);
         res = check_user_file (path,
-                               user->uid,
-                               MAX_FILE_SIZE,
-                               RELAX_GROUP,
-                               RELAX_OTHER);
+                               MAX_FILE_SIZE);
         if (res) {
                 retval = gdk_pixbuf_new_from_file_at_size (path,
                                                            icon_size,
                                                            icon_size,
                                                            NULL);
         } else {
+                g_debug ("Could not access face icon %s", path);
                 retval = NULL;
         }
         g_free (path);
 
-        /* Next, try "~/.face.icon" */
-        if (retval == NULL) {
-                path = g_build_filename (user->home_dir,
-                                         ".face.icon",
-                                         NULL);
-                res = check_user_file (path,
-                                       user->uid,
-                                       MAX_FILE_SIZE,
-                                       RELAX_GROUP,
-                                       RELAX_OTHER);
-                if (res) {
-                        retval = gdk_pixbuf_new_from_file_at_size (path,
-                                                                   icon_size,
-                                                                   icon_size,
-                                                                   NULL);
-                } else {
-                        retval = NULL;
-                }
-
-                g_free (path);
-        }
-
-        /* Still nothing, try the user's personal GDM config */
-        if (retval == NULL) {
-                path = g_build_filename (user->home_dir,
-                                         ".gnome",
-                                         "gdm",
-                                         NULL);
-                res = check_user_file (path,
-                                       user->uid,
-                                       MAX_FILE_SIZE,
-                                       RELAX_GROUP,
-                                       RELAX_OTHER);
-                if (res) {
-                        GKeyFile *keyfile;
-                        char     *icon_path;
-
-                        keyfile = g_key_file_new ();
-                        g_key_file_load_from_file (keyfile,
-                                                   path,
-                                                   G_KEY_FILE_NONE,
-                                                   NULL);
-
-                        icon_path = g_key_file_get_string (keyfile,
-                                                           "face",
-                                                           "picture",
-                                                           NULL);
-                        res = check_user_file (icon_path,
-                                               user->uid,
-                                               MAX_FILE_SIZE,
-                                               RELAX_GROUP,
-                                               RELAX_OTHER);
-                        if (icon_path && res) {
-                                retval = gdk_pixbuf_new_from_file_at_size (path,
-                                                                           icon_size,
-                                                                           icon_size,
-                                                                           NULL);
-                        } else {
-                                retval = NULL;
-                        }
-
-                        g_free (icon_path);
-                        g_key_file_free (keyfile);
-                } else {
-                        retval = NULL;
-                }
-
-                g_free (path);
-        }
-
         return retval;
 }
 
@@ -1114,7 +1097,7 @@ gdm_user_render_icon (GdmUser   *user,
 
         path = NULL;
 
-        pixbuf = render_icon_from_home (user, icon_size);
+        pixbuf = render_icon_from_cache (user, icon_size);
         if (pixbuf != NULL) {
                 goto out;
         }
@@ -1122,16 +1105,14 @@ gdm_user_render_icon (GdmUser   *user,
         /* Try ${GlobalFaceDir}/${username} */
         path = g_build_filename (GLOBAL_FACEDIR, user->user_name, NULL);
         res = check_user_file (path,
-                               user->uid,
-                               MAX_FILE_SIZE,
-                               RELAX_GROUP,
-                               RELAX_OTHER);
+                               MAX_FILE_SIZE);
         if (res) {
                 pixbuf = gdk_pixbuf_new_from_file_at_size (path,
                                                            icon_size,
                                                            icon_size,
                                                            NULL);
         } else {
+                g_debug ("Could not access global face icon %s", path);
                 pixbuf = NULL;
         }
 
@@ -1145,16 +1126,14 @@ gdm_user_render_icon (GdmUser   *user,
         path = g_build_filename (GLOBAL_FACEDIR, tmp, NULL);
         g_free (tmp);
         res = check_user_file (path,
-                               user->uid,
-                               MAX_FILE_SIZE,
-                               RELAX_GROUP,
-                               RELAX_OTHER);
+                               MAX_FILE_SIZE);
         if (res) {
                 pixbuf = gdk_pixbuf_new_from_file_at_size (path,
                                                            icon_size,
                                                            icon_size,
                                                            NULL);
         } else {
+                g_debug ("Could not access global face icon %s", path);
                 pixbuf = NULL;
         }
         g_free (path);
